#coding:utf-8

class ImageNotFound(Exception):
    """
    Raised when no Image matching the query was found.
    """

class ReleaseNotFound(Exception):
    """
    Raised when the latest Release could not be identified.
    """

class ServiceUnavailable(Exception):
    """
    Raised when the Ubuntu Cloud Images service is not available
    """



#coding:utf-8
import datetime


class BaseImage(object):
    """
    A base image model, as can be searched for.

    :ivar release: The Ubuntu release codename for this Image.
    :ivar platform: Set to server for Cloud Images.
    :ivar instance_type: The instance type supported by this Image (ebs or instance-store).
    :ivar architecture: The architecture for this Image (i386 or amd64)
    :ivar region: The region for this Image (us-east-1, etc.)
    :ivar virtualization: The virtualization technology for this Image (paravirtual of hvm)
    """
    def __init__(self, release, platform, instance_type, architecture, region, virtualization):
        self.release = release
        self.platform = platform
        self.instance_type = instance_type
        self.architecture = architecture
        self.region = region
        self.virtualization = virtualization

    def __str__(self):
        return "{0}@{1} ({2} {3} {4})".format(self.release, self.region, self.architecture, self.instance_type,
                                              self.virtualization)

    def __repr__(self):
        return "<{0}: {1}>".format(self.__class__.__name__, self)


class SearchImage(BaseImage):
    """
    An Image used to search for other images.
    """
    def matches(self, image):
        """
        Checks whether an image matches this search image.

        :param image: The Image to compare to the SearchImage
        :type image: BaseImage

        :returns: Whether the Image matches
        :rtype: bool
        """
        for attr in ["release", "platform", "instance_type", "architecture", "region", "virtualization"]:
            if getattr(self, attr) != getattr(image, attr):
                return False
        return True


class Image(BaseImage):
    """
    An image as presented in CloudImages.

    :ivar date: The date this Image was released
    :ivar stability: The stability of this Image (release or devel)
    :ivar ami_id: The AMI ID for this Image
    :ivar aki_id: The AKI ID for this Image
    """
    def __init__(self, release, platform, stability, date, instance_type, architecture, region, ami_id, aki_id,
                 _, virtualization):

        super(Image, self).__init__(release, platform, instance_type, architecture, region, virtualization)

        # Sometimes the date has ".1", so we need to remove it.
        self.date = datetime.datetime.strptime(date.split('.', 1)[0], "%Y%m%d")
        self.stability = stability
        self.ami_id = ami_id
        self.aki_id = aki_id
        self._ = _

    def __str__(self):
        return "{0}@{1}: {2} ({3} {4} {5})".format(self.release, self.region, self.date, self.architecture,
                                               self.instance_type, self.virtualization)



#coding:utf-8
from __future__ import print_function

import sys
import argparse

from ubuntufinder.actions import LATEST, find_image
from ubuntufinder.exceptions import ImageNotFound


def main():
    parser = argparse.ArgumentParser("ubuntufinder", description="An utility tool to find the latest Ubuntu EC2 AMI IDs")
    parser.add_argument("region", help="AWS Region to return an image ID for.")
    parser.add_argument("-r", "--release", default=LATEST, help="Release codename. Defaults to the latest release.")
    parser.add_argument("-a", "--architecture", default="amd64", help="Architecture. `amd64` or `i386`. Defauts to `amd64`.")
    parser.add_argument("-i", "--image-type", default="ebs", help="AWS image type. `ebs` or `instance-store`. Defaults to `ebs`.")
    parser.add_argument("-v", "--virtualization", default="paravirtual", help="Virtualization type. `paravirtual` or `hvm`. Defaults to `paravirtual`.")
    parser.add_argument('--instance-type')  # Legacy argument name

    args = parser.parse_args()

    # Legacy argument handling
    if args.instance_type is not None:
        print("WARNING: `--instance-type` is deprecated. Use `--image-type` instead", file=sys.stderr)
        image_type = args.instance_type
    else:
        image_type = args.image_type

    # Actual image location
    try:
        image = find_image(args.region, args.release, args.architecture, image_type, args.virtualization)
    except ImageNotFound:
        print("ERROR: No image found!", file=sys.stderr)
        sys.exit(1)

    print(image.ami_id)

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import csv
import StringIO

import requests

from ubuntufinder.exceptions import ImageNotFound, ReleaseNotFound, ServiceUnavailable
from ubuntufinder.models import SearchImage, Image


CLOUD_IMAGES_SERVER = "https://cloud-images.ubuntu.com/query" # The server that serves the Cloud Images
CLOUD_IMAGES_LIST_FILE = "released.current.txt"
CLOUD_IMAGES_RELEASES_FILE = "released.latest.txt"
CLOUD_IMAGES_CSV_SEPARATOR = "\t"

LATEST = "latest"  # Code to search for the latest release


def _open_csv_from_url(url, _session=None):
    """
    :param url: The URL to open

    :returns: A CSV reader from the data at URL
    :rtype: :class:`csv.reader`
    """

    session = _session or requests.Session()
    try:
        res = session.get(url)
    except requests.RequestException as e:
        raise ServiceUnavailable(e)
    return csv.reader(StringIO.StringIO(res.text), delimiter=CLOUD_IMAGES_CSV_SEPARATOR)


def _find_latest_release(_session):
    """
    :returns: The latest stable (non-devel) Ubuntu release
    :rtype: str
    """

    url = "/".join([CLOUD_IMAGES_SERVER, CLOUD_IMAGES_RELEASES_FILE])
    reader = _open_csv_from_url(url, _session)

    latest_release = None
    for release, platform, status, date in reader:
        if status == "release":
            latest_release = release

    if latest_release is None:
        raise ReleaseNotFound()

    return latest_release


def _list_images(release, _session=None):
    """
    :rtype: list of ubuntufinder.models.Image
    """

    url = "/".join([CLOUD_IMAGES_SERVER, release, "server", CLOUD_IMAGES_LIST_FILE])
    reader = _open_csv_from_url(url, _session)
    return [Image(*entry) for entry in reader]


def _find_image(region, release, architecture, instance_type, virtualization, _session=None):
    """
    :rtype: ubuntufinder.models.Image
    """

    search_image = SearchImage(release, "server", instance_type, architecture, region, virtualization)

    for image in _list_images(release, _session):
        if search_image.matches(image):
            return image

    raise ImageNotFound()


def find_image(region, release=LATEST, architecture="amd64", instance_type="ebs", virtualization="paravirtual",
               _session=None):
    """
    Return a full image specification according to the query parameters

    :param region: The AWS region code to locate the Image in
    :type region: :class:`str`

    :param release: The codename of the Ubuntu Release to locate. Defaults to the latest release
    :type release: :class:`str`

    :param architecture: The CPU Architecture to find the image for. Defaults to amd64
    :type architecture: :class:`str`

    :param instance_type: The Instance type to find the image for (ebs or instance-store). Defaults to ebs.
    :type instance_type: :class:`str`

    :param virtualization: The virtualization technology to find the image for (paravirtual or hvm).
    :type virtualization: :class:`str`

    :returns: An Image corresponding to your search
    :rtype: :class:`ubuntufinder.models.Image`

    :raises: :class:`ubuntufinder.exceptions.ImageNotFound` if no match is found.
    :raises: :class:`ubuntufinder.exceptions.ServiceUnavailable` if Cloud Images can't be accessed.
    """

    if release == LATEST:
        release = _find_latest_release(_session)

    return _find_image(region, release, architecture, instance_type, virtualization, _session)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')
execfile('ubuntufinder/version.py') # Load version

setup(
    name='ubuntufinder',
    version=__version__,
    description='An utility package to locate the latest Ubuntu AMIs.',
    long_description=readme + '\n\n' + history,
    author='Thomas Orozco',
    author_email='thomas@orozco.fr',
    url='https://github.com/krallin/ubuntufinder',
    packages=[
        'ubuntufinder',
    ],
    package_dir={'ubuntufinder': 'ubuntufinder'},
    entry_points={
        'console_scripts': [
            'ubuntufinder = ubuntufinder.cli:main',
        ]
    },
    include_package_data=True,
    install_requires=[
        'requests',
    ],
    license='BSD',
    zip_safe=False,
    keywords='ubuntufinder',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
    ],
    test_suite='tests',
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'Thomas Orozco'
__email__ = 'thomas@orozco.fr'

from ubuntufinder.version import __version__

from ubuntufinder.actions import find_image
from ubuntufinder.models import Image
from ubuntufinder.exceptions import ImageNotFound, ReleaseNotFound


__all__ = ['find_image', 'Image', 'ImageNotFound', 'ReleaseNotFound']
